unit main;

{$MODE Delphi}

interface

uses
  ModemEmulator{, Windows},
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, statmach, ExtCtrls, Buttons, StdCtrls;

type
  TForm4 = class(TForm)
    StateMachine1: TStateMachine;
    Panel2: TPanel;
    StateStart: TStateStart;
    StateInit: TStateNode;
    StateReset: TStateNode;
    StateWaitCall: TStateNode;
    StatePickup: TStateNode;
    StateGreetHello: TStateNode;
    StateLinkHangupIn: TStateLink;
    StatePromptItem: TStateNode;
    StateBooleanItemValid: TStateBoolean;
    StateTransitionItemReject: TStateTransition;
    StateTransitionAddressStreetReject: TStateTransition;
    StateBooleanMore: TStateBoolean;
    StateLinkGoodbye: TStateLink;
    StateBooleanAddressStreetValid: TStateBoolean;
    StatePromptAddressStreet: TStateNode;
    StateVerify: TStateNode;
    StatePromptAddressNumber: TStateNode;
    StateBooleanOrderValid: TStateBoolean;
    StateOrderSave: TStateNode;
    StateGreetThankYou: TStateNode;
    StateOrderCancel: TStateNode;
    StateGreetGoodbye: TStateNode;
    StateLinkCancelToGoodbye: TStateLink;
    StateLinkReset: TStateLink;
    StateLinkThanksToReset: TStateLink;
    ButtonStart: TButton;
    ButtonStop: TButton;
    StateBooleanAddressNumberValid: TStateBoolean;
    StateTransitionAddressNumberReject: TStateTransition;
    procedure FormShow(Sender: TObject);
    procedure ButtonStartClick(Sender: TObject);
    procedure StateMachine1Start(Sender: TObject);
    procedure StateMachine1Stop(Sender: TObject);
    procedure ButtonStopClick(Sender: TObject);
    procedure StateInitEnterState(Sender: TObject);
    procedure StateWaitCallEnterState(Sender: TObject);
    procedure StateMachine1ChangeState(Sender: TStateMachine; FromState,
      ToState: TStateControl);
    procedure StatePickupEnterState(Sender: TObject);
    procedure StateGreetHelloEnterState(Sender: TObject);
    procedure StatePromptItemEnterState(Sender: TObject);
    procedure StateBooleanItemValidEnterState(Sender: TStateBoolean;
      var Result: Boolean);
    procedure StateTransitionItemRejectTransition(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure StateResetEnterState(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure StateBooleanMoreEnterState(Sender: TStateBoolean;
      var Result: Boolean);
    procedure StateTransitionAddressStreetRejectTransition(Sender: TObject);
    procedure StatePromptAddressStreetEnterState(Sender: TObject);
    procedure StatePromptAddressNumberEnterState(Sender: TObject);
    procedure StateBooleanAddressStreetValidEnterState(Sender: TStateBoolean;
      var Result: Boolean);
    procedure StateTransitionAddressNumberRejectTransition(Sender: TObject);
    procedure StateBooleanAddressNumberValidEnterState(Sender: TStateBoolean;
      var Result: Boolean);
    procedure StateVerifyEnterState(Sender: TObject);
    procedure StateBooleanOrderValidEnterState(Sender: TStateBoolean;
      var Result: Boolean);
    procedure StateOrderCancelEnterState(Sender: TObject);
    procedure StateOrderSaveEnterState(Sender: TObject);
    procedure StateGreetThankYouEnterState(Sender: TObject);
    procedure StateGreetGoodbyeEnterState(Sender: TObject);
  private
    FModem: TFormModemEmulator;
    FLastChoice: string;
    FItems: TList;
    FAddressStreet: string;
    FAddressNumber: string;
  protected
    procedure OnIncomingCall(Sender: TObject);
    procedure OnHangup(Sender: TObject);
  public
  end;


var
  Form4: TForm4;

const
  sAnyKey = '0123456789*#';

implementation

{$R *.lfm}

(*******************************************************************************
**
**			Form/Application Event Handlers
**
*******************************************************************************)
procedure TForm4.FormCreate(Sender: TObject);
begin
  FModem := TFormModemEmulator.Create(Self);
  FModem.OnCallIncoming := OnIncomingCall;
  FModem.OnHangup := OnHangup;

  FItems := TList.Create;
end;

procedure TForm4.FormDestroy(Sender: TObject);
begin
  FItems.Free;
end;

procedure TForm4.FormShow(Sender: TObject);
begin
  FModem.Left := Left+Width;
  FModem.Top := Top;
  FModem.Show;
  StateMachine1.Show;
end;

procedure TForm4.ButtonStartClick(Sender: TObject);
begin
  ButtonStart.Enabled := False;

  StateMachine1.State := StateStart;
  StateMachine1.Execute;
end;

procedure TForm4.ButtonStopClick(Sender: TObject);
begin
  ButtonStop.Enabled := False;

  StateMachine1.Stop;
  FModem.Stop;

  FModem.ReleaseLine;
end;

(*******************************************************************************
**
**			Modem Event Handlers
**
*******************************************************************************)
procedure TForm4.OnHangup(Sender: TObject);
begin
  StateMachine1.State := StateLinkHangupIn;

  if (not StateMachine1.Active) then
    StateMachine1.Execute;
end;

procedure TForm4.OnIncomingCall(Sender: TObject);
begin
  StateMachine1.State := StatePickup;

  if (not StateMachine1.Active) then
    StateMachine1.Execute;
end;

(*******************************************************************************
**
**			State Machine Event Handlers
**
*******************************************************************************)
procedure TForm4.StateMachine1ChangeState(Sender: TStateMachine; FromState,
  ToState: TStateControl);
begin
  Sleep(200);
end;

procedure TForm4.StateMachine1Start(Sender: TObject);
begin
  ButtonStart.Enabled := False;
  ButtonStop.Enabled := True;
end;

procedure TForm4.StateMachine1Stop(Sender: TObject);
begin
  ButtonStart.Enabled := True;
  ButtonStop.Enabled := False;
end;

(*******************************************************************************
**
**			State Transition Event Handlers
**
*******************************************************************************)
procedure TForm4.StateInitEnterState(Sender: TObject);
begin
  // Prepare voice/respose to receive calls
  FModem.AllocateLine(0);
end;

procedure TForm4.StateWaitCallEnterState(Sender: TObject);
begin
  // Wait for incoming call
  FModem.DetectRing(1);
end;

procedure TForm4.StatePickupEnterState(Sender: TObject);
begin
  // Call detected - pickup
  FModem.Pickup;
end;

procedure TForm4.StateGreetGoodbyeEnterState(Sender: TObject);
begin
  FModem.Purge;
  FModem.Speak('Good bye.');
end;

procedure TForm4.StateGreetHelloEnterState(Sender: TObject);
begin
  FModem.Speak('Welcome to Pizzatron 2000.');
end;

procedure TForm4.StatePromptAddressNumberEnterState(Sender: TObject);
begin
  FModem.Purge;
  FModem.Speak('Please enter the street number of the delivery address followed by *', sAnyKey);
  FModem.Speak('or press # to go back', sAnyKey);
  FLastChoice := '';
  repeat
    FLastChoice := FLastChoice + FModem.GetDigits('0123456789', '*#', 30000);
  until (not FModem.OffHook) or (FLastChoice = '') or (FModem.LastDigit in ['*', '#']);

  if (FModem.LastDigit = '#') then
    StateMachine1.State := StatePromptAddressStreet;

  FAddressNumber := FLastChoice;
end;

procedure TForm4.StatePromptAddressStreetEnterState(Sender: TObject);
begin
  FModem.Purge;
  FModem.Speak('Please enter the street name of the delivery address followed by *', sAnyKey);
  FModem.Speak('or press # to go back', sAnyKey);
  FLastChoice := '';
  repeat
    FLastChoice := FLastChoice + FModem.GetDigits('23456789', '*#', 30000);
  until (not FModem.OffHook) or (FLastChoice = '') or (FModem.LastDigit in ['*', '#']);

  if (FModem.LastDigit = '#') then
    StateMachine1.State := StatePromptItem;

  FAddressStreet := FLastChoice;
end;

procedure TForm4.StatePromptItemEnterState(Sender: TObject);
begin
  FModem.Purge;
  FModem.Speak('Please enter the number of the pizza you wish to order followed by *', sAnyKey);
  FModem.Speak('or press # to complete your order', sAnyKey);
  FLastChoice := '';
  repeat
    FLastChoice := FLastChoice + FModem.GetDigits('0123456789', '*#', 30000);
  until (not FModem.OffHook) or (FLastChoice = '') or (FModem.LastDigit in ['*', '#']);
end;

procedure TForm4.StateBooleanItemValidEnterState(Sender: TStateBoolean;
  var Result: Boolean);
var
  ItemNumber: integer;
begin
  ItemNumber := StrToIntDef(FLastChoice, 0);
  Result := (ItemNumber >= 1) and (ItemNumber <= 22);

  // Save order
  if (Result) then
    FItems.Add(pointer(ItemNumber));
end;

procedure TForm4.StateBooleanMoreEnterState(Sender: TStateBoolean;
  var Result: Boolean);
begin
  Result := (FModem.LastDigit <> '#');
end;

procedure TForm4.StateBooleanOrderValidEnterState(Sender: TStateBoolean;
  var Result: Boolean);
begin
  FModem.Speak('Please press * to confirm your order', sAnyKey);
  FModem.Speak('or press # to cancel the order', sAnyKey);
  FLastChoice := '';
  repeat
    FLastChoice := FLastChoice + FModem.GetDigits('', '*#', 30000);
  until (not FModem.OffHook) or (FLastChoice = '') or (FModem.LastDigit in ['*', '#']);

  Result := (FModem.LastDigit = '*');
end;

procedure TForm4.StateBooleanAddressNumberValidEnterState(Sender: TStateBoolean;
  var Result: Boolean);
begin
  // Validate the street number through a directory lookup
  // ...
  Result := (FAddressNumber <> '');
end;

procedure TForm4.StateBooleanAddressStreetValidEnterState(Sender: TStateBoolean;
  var Result: Boolean);
begin
  // Validate the street name through a directory lookup
  // ...
  Result := (FAddressStreet <> '');
end;

procedure TForm4.StateTransitionItemRejectTransition(Sender: TObject);
begin
  FModem.Speak('The number you entered was not a valid choice.');
  FModem.Speak('Valid choices are 1 through 22.', sAnyKey);
  FModem.Speak('Remember to press the * sign once you have entered the number of the pizza you wish to order', sAnyKey);
end;

procedure TForm4.StateVerifyEnterState(Sender: TObject);
var
  i: integer;
begin
  FModem.Purge;
  FModem.Speak('Please take a moment to verify your order.');
  FModem.Speak('You have ordered:');
  for i := 0 to FItems.Count-1 do
    FModem.Speak('Pizza number '+IntToStr(integer(FItems[i])));
  FModem.Speak('Your order will be delivered to :');
  FModem.Speak(FAddressNumber+', '+FAddressStreet);
end;

procedure TForm4.StateTransitionAddressNumberRejectTransition(Sender: TObject);
begin
  FModem.Speak('Number '+FAddressNumber+' could not be found on '+FAddressStreet);
  FModem.Speak('Remember to press the * sign once you have entered the street number', sAnyKey);
end;

procedure TForm4.StateTransitionAddressStreetRejectTransition(Sender: TObject);
begin
  FModem.Speak('The street name was not recognized.');
  FModem.Speak('Remember to press the * sign once you have entered the street name', sAnyKey);
end;

procedure TForm4.StateOrderCancelEnterState(Sender: TObject);
begin
  FModem.Purge;
  FModem.Speak('Your order has been canceled.');
end;

procedure TForm4.StateOrderSaveEnterState(Sender: TObject);
begin
  // Do something to order here
  // ...
end;

procedure TForm4.StateGreetThankYouEnterState(Sender: TObject);
begin
  FModem.Purge;
  FModem.Speak('Thank you for your order.');
  FModem.Speak('Good bye.');
end;

procedure TForm4.StateResetEnterState(Sender: TObject);
begin
  FModem.Stop;
  FModem.Reset;
  FItems.Clear;
end;

end.
